summaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorAtish Patra <atishp@rivosinc.com>2022-03-24 16:20:57 +0800
committerminda.chen <minda.chen@starfivetech.com>2023-01-03 14:26:17 +0800
commitcd2df9c82881b630f632fcc85a356b63e52683e7 (patch)
tree59f1b11790e7ecbe629f1e146ad3e4768a443736
parente80193abdfc06372234496e3d4b7b0cba915bcd9 (diff)
downloadlinux-starfive-cd2df9c82881b630f632fcc85a356b63e52683e7.tar.gz
linux-starfive-cd2df9c82881b630f632fcc85a356b63e52683e7.tar.bz2
linux-starfive-cd2df9c82881b630f632fcc85a356b63e52683e7.zip
RISC-V: Improve /proc/cpuinfo output for ISA extensions
Currently, the /proc/cpuinfo outputs the entire riscv,isa string which is not ideal when we have multiple ISA extensions present in the ISA string. Some of them may not be enabled in kernel as well. Parse only the enabled ISA extension and print them in a separate row. Signed-off-by: Atish Patra <atishp@rivosinc.com>
-rw-r--r--arch/riscv/include/asm/hwcap.h7
-rw-r--r--arch/riscv/kernel/cpu.c46
2 files changed, 51 insertions, 2 deletions
diff --git a/arch/riscv/include/asm/hwcap.h b/arch/riscv/include/asm/hwcap.h
index 170bd80da520..691fc9c8099b 100644
--- a/arch/riscv/include/asm/hwcap.h
+++ b/arch/riscv/include/asm/hwcap.h
@@ -54,6 +54,13 @@ enum riscv_isa_ext_id {
RISCV_ISA_EXT_ID_MAX = RISCV_ISA_EXT_MAX,
};
+struct riscv_isa_ext_data {
+ /* Name of the extension displayed to userspace via /proc/cpuinfo */
+ char uprop[RISCV_ISA_EXT_NAME_LEN_MAX];
+ /* The logical ISA extension ID */
+ unsigned int isa_ext_id;
+};
+
unsigned long riscv_isa_extension_base(const unsigned long *isa_bitmap);
#define riscv_isa_extension_mask(ext) BIT_MASK(RISCV_ISA_EXT_##ext)
diff --git a/arch/riscv/kernel/cpu.c b/arch/riscv/kernel/cpu.c
index 6d59e6906fdd..4d51a4296fc2 100644
--- a/arch/riscv/kernel/cpu.c
+++ b/arch/riscv/kernel/cpu.c
@@ -6,6 +6,7 @@
#include <linux/init.h>
#include <linux/seq_file.h>
#include <linux/of.h>
+#include <asm/hwcap.h>
#include <asm/smp.h>
/*
@@ -61,12 +62,50 @@ int riscv_of_parent_hartid(struct device_node *node)
}
#ifdef CONFIG_PROC_FS
+#define __RISCV_ISA_EXT_DATA(UPROP, EXTID) \
+ { \
+ .uprop = #UPROP, \
+ .isa_ext_id = EXTID, \
+ }
+
+static struct riscv_isa_ext_data isa_ext_arr[] = {
+ __RISCV_ISA_EXT_DATA("", RISCV_ISA_EXT_MAX),
+};
+
+static void print_isa_ext(struct seq_file *f)
+{
+ struct riscv_isa_ext_data *edata;
+ int i = 0, arr_sz;
+
+ arr_sz = ARRAY_SIZE(isa_ext_arr) - 1;
+
+ /* No extension support available */
+ if (arr_sz <= 0)
+ return;
+
+ seq_puts(f, "isa-ext\t\t: ");
+ for (i = 0; i <= arr_sz; i++) {
+ edata = &isa_ext_arr[i];
+ if (!__riscv_isa_extension_available(NULL, edata->isa_ext_id))
+ continue;
+ seq_printf(f, "%s ", edata->uprop);
+ }
+ seq_puts(f, "\n");
+}
static void print_isa(struct seq_file *f, const char *isa)
{
- /* Print the entire ISA as it is */
+ char *ext_start;
+ int isa_len = strlen(isa);
+ int base_isa_len = isa_len;
+
+ ext_start = strnchr(isa, isa_len, '_');
+ if (ext_start)
+ base_isa_len = isa_len - strlen(ext_start);
+
+ /* Print only the base ISA as it is */
seq_puts(f, "isa\t\t: ");
- seq_write(f, isa, strlen(isa));
+ seq_write(f, isa, base_isa_len);
seq_puts(f, "\n");
}
@@ -114,6 +153,9 @@ static int c_show(struct seq_file *m, void *v)
print_isa(m, isa);
if (!of_property_read_string(node, "mmu-type", &mmu))
print_mmu(m, mmu);
+
+ print_isa_ext(m);
+
if (!of_property_read_string(node, "compatible", &compat)
&& strcmp(compat, "riscv"))
seq_printf(m, "uarch\t\t: %s\n", compat);